package com.smg.persistwork.connexio.bd;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import com.smg.persistwork.exception.PersistWorkException;

/**
 * ConnexioBD: classe abstracta que s'encarrega de les connexions amb la base de
 * dades i d'executar les sentncies.
 * 
 * @version 1.0
 * @author Sergio Maeso
 * 
 */
public abstract class ConnexioBD {

	/**
	 * Cadena de connexi de la base de dades.
	 */
	private String cadenaConnexio;
	/**
	 * Usuari per a connectar a la base de dades.
	 */
	private String usuari;
	/**
	 * Contrasenya per a connectar a la base de dades.
	 */
	private String contrasenya;
	/**
	 * Connexi amb la base de dades.
	 */
	private Connection connexio;

	public static String BD_ORACLE = "ORACLE";
	public static String BD_MYSQL = "MYSQL";

	/**
	 * Constructor que informa dels valors necessaris per a realitzar connexions
	 * amb la base de dades.
	 * 
	 * @param usuari
	 * @param contrasenya
	 * @param cadenaConnexio
	 */
	public ConnexioBD(String cadenaConnexio, String usuari, String contrasenya) {
		super();
		this.usuari = usuari;
		this.contrasenya = contrasenya;
		this.cadenaConnexio = cadenaConnexio;
	}

	/**
	 * Mtode per a obrir una connexi amb la base de dades.
	 * 
	 * @throws PersistWorkException
	 */
	public void creaConnexio() throws PersistWorkException {

		try {
			inicialitzacio();

			this.connexio = DriverManager.getConnection(cadenaConnexio, usuari,
					contrasenya);

		} catch (SQLException e) {		
			throw new PersistWorkException(PersistWorkException.BD_1,
					PersistWorkException.BD1_TXT);
		}

	}

	/**
	 * Mtode pera tancar la connexi oberta.
	 * 
	
	 * @throws PersistWorkException
	 */
	public void tancaConnexio() throws PersistWorkException {
		try {
			if (connexio != null) {

				if (!connexio.isClosed()) {
					if (!connexio.getAutoCommit()) {
						connexio.commit();
					}
					connexio.close();
				}

			}

		} catch (SQLException e) {
		
			try {
				if (!connexio.getAutoCommit()) {
					connexio.rollback();
				}
			} catch (SQLException e1) {			
				throw new PersistWorkException(PersistWorkException.BD_2,
						PersistWorkException.BD2_TXT);
			}
		} finally {
			connexio = null;
		}

	}

	/**
	 * Mtode per a fer marxa enrera de les sentncies executades. Per a que
	 * sigui efectiu ha d'estar oberta una transacci.
	 * 	
	 * @throws PersistWorkException
	 */
	public void rollback() throws PersistWorkException {
		try {
			if (connexio != null) {

				if (!connexio.isClosed()) {
					connexio.rollback();
					connexio.close();
				}

			}

		} catch (SQLException e) {

			e.printStackTrace();
			try {
				if (!connexio.getAutoCommit()) {
					connexio.rollback();
				}
			} catch (SQLException e1) {
			
				throw new PersistWorkException(PersistWorkException.BD_3,
						PersistWorkException.BD3_TXT);
			}
		} finally {
			connexio = null;
		}

	}

	/**
	 * Mtode per a inicialitzar una transacci de base de dades.
	 * 
	
	 * @throws PersistWorkException
	 */
	public void iniciaTransaccio() throws PersistWorkException {
		try {
			if (connexio == null) {
				creaConnexio();
			}
			connexio.setAutoCommit(false);

		} catch (SQLException e) {

			throw new PersistWorkException(PersistWorkException.BD_4,
					PersistWorkException.BD4_TXT);
		}

	}

	/**
	 * Mtode que finalitza la transacci amb la base de dades i tanca la
	 * connexi.
	 * 
	
	 * @throws PersistWorkException
	 */
	public void finalitzaTransaccio() throws PersistWorkException {
		tancaConnexio();
	}

	/*
	 * Mtode que finalitza l'objecte, tancant la connexi si fos necessari.
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		tancaConnexio();
		super.finalize();
	}

	/**
	 * Executa una sentncia SQL sense retornar dades.
	 * 
	 * @param sentencia
	 * @return resultat execuci
	 * @throws PersistWorkException
	 */
	public boolean executaSentencia(String sentencia)
			throws PersistWorkException {
		if (connexio == null) {	
			throw new PersistWorkException(PersistWorkException.BD_8,
					PersistWorkException.BD8_TXT);

		}
		Statement st;
		try {
			st = connexio.createStatement();

			boolean res2 = st.execute(sentencia);
			return res2;
		} catch (SQLException e) {			
			throw new PersistWorkException(PersistWorkException.BD_5,
					PersistWorkException.BD5_TXT +": "+sentencia);
		} catch (Exception e) {
			throw new PersistWorkException(PersistWorkException.BD_5,
					PersistWorkException.BD5_TXT +": "+sentencia);
		}

	}

	/**
	 * Executa una sentencia retornat les dades.
	 * 
	 * @param sentencia
	 * @return ResultSet
	 * @throws PersistWorkException
	 */
	public ResultSet executaSentenciaDades(String sentencia)
			throws PersistWorkException {
		if (connexio == null) {
	
			throw new PersistWorkException(PersistWorkException.BD_8,
					PersistWorkException.BD8_TXT);
		}
		Statement st;
		try {
			st = connexio.createStatement();
			st.execute(sentencia);
			ResultSet resultat2 = st.getResultSet();
			return resultat2;
		} catch (SQLException e) {
			throw new PersistWorkException(PersistWorkException.BD_6,
					PersistWorkException.BD6_TXT+": "+sentencia);
		} catch (Exception e) {
			throw new PersistWorkException(PersistWorkException.BD_6,
					PersistWorkException.BD6_TXT+": "+sentencia);
		}

	}

	/**
	 * Mtode que inicialitzar la connexi de base de dades, dependr del tipus
	 * de motor de base de daes.
	 * 
	 */
	protected abstract void inicialitzacio() throws PersistWorkException;

	/**
	 * Retorna la connexi que s'ha creat.
	 * 
	 * @return connexi
	 */
	protected Connection getConnexio() {
		return connexio;
	}

	/**
	 * Executa una sentncia amb preparedstatement passant diferents valors en
	 * una llista d'objectes.
	 * 
	 * @param sentencia
	 * @param valors
	 * @return  resultat execuci
	 * @throws PersistWorkException
	 */
	public boolean executaSentencia(String sentencia, List<Object> valors)
			throws PersistWorkException {

		PreparedStatement st;
		try {
			st = getStatement(sentencia, valors);

			return st.execute();

		} catch (SQLException e) {
			
			e.printStackTrace();
			throw new PersistWorkException(PersistWorkException.BD_7,
					PersistWorkException.BD7_TXT+": "+sentencia);
		} catch (Exception e) {
			
			e.printStackTrace();
			throw new PersistWorkException(PersistWorkException.BD_7,
					PersistWorkException.BD7_TXT+": "+sentencia);
		}

	}

	/**
	 * Retornar el preparedstatement per a realitzar la consulta, segons la
	 * sentncia i els valor que s'han passat per parmetre.
	 * 
	 * @param sentencia
	 * @param valors
	 * @return PreparedStatement
	 * @throws PersistWorkException
	 */
	private PreparedStatement getStatement(String sentencia, List<Object> valors)
			throws PersistWorkException {
		if (getConnexio() == null) {
			throw new PersistWorkException(PersistWorkException.BD_8,
					PersistWorkException.BD8_TXT);

		}

		PreparedStatement st = null;

		try {
			st = getConnexio().prepareStatement(sentencia);

			int par = 1;
			for (Object val : valors) {

				if (val instanceof String) {
					st.setString(par, (String) val);
				}

				if (val instanceof Integer) {
					st.setInt(par, (Integer) val);
				}

				if (val instanceof Long) {
					st.setLong(par, (Long) val);
				}

				if (val instanceof BigDecimal) {

					st.setBigDecimal(par, (BigDecimal) val);
				}

				if (val instanceof Double) {
					st.setDouble(par, (Double) val);
				}

				if (val instanceof Float) {
					st.setFloat(par, (Float) val);
				}

				if (val instanceof Date) {
					java.sql.Date sqlDate = new java.sql.Date(
							((Date) val).getTime());
					st.setDate(par, sqlDate);
				}

				if (val instanceof GregorianCalendar) {
					java.sql.Date sqlDate = new java.sql.Date(
							((GregorianCalendar) val).getTime().getTime());
					st.setDate(par, sqlDate);
				}
				par++;

			}
		} catch (SQLException e) {
			
			e.printStackTrace();
			throw new PersistWorkException(PersistWorkException.BD_9,
					PersistWorkException.BD9_TXT+": "+sentencia);
		}
		return st;

	}

	/**
	 * Executa una sentncia i retorna les dades, es passen valors per a
	 * filtrar com a parmetre en una llista d'objectes.
	 * @param sentencia
	 * @param valors
	 * @return ResultSet
	 * @throws PersistWorkException
	 */
	public ResultSet executaSentenciaDades(String sentencia, List<Object> valors)
			throws PersistWorkException {
		try {
			PreparedStatement st = getStatement(sentencia, valors);
			return st.executeQuery();
		} catch (SQLException e) {
			
			throw new PersistWorkException(PersistWorkException.BD_10,
					PersistWorkException.BD10_TXT+": "+sentencia);
		} catch (Exception e) {
			
			throw new PersistWorkException(PersistWorkException.BD_10,
					PersistWorkException.BD10_TXT+": "+sentencia);
		}
	}

}
