package com.smg.persistwork.model.mapeig;

import com.smg.persistwork.connexio.bd.ConnexioBD;
import com.smg.persistwork.exception.PersistWorkException;
import com.smg.persistwork.nucli.ConfiguracioEntorn;

/**
 * Utilitat que serveix per a retornar la equivalncia d'una columna de la base
 * de dades segons el tipus de camp de l'objecte.
 * 
 * @author Sergio Maeso
 * @version 1.0
 * 
 */
public abstract class EquivalenciaColumna {

	protected EquivalenciaColumna() {
		super();
	}

	/**
	 * Retornar el tipus de columna segons el tipus de camp
	 * 
	 * @param tipus
	 *            de camp
	 * @return tipus de columna
	 * @throws PersistWorkException
	 */
	public final String getTipusColumna(String tipus)
			throws PersistWorkException {

		if (isCampText(tipus)) {
			return getColumnaText();
		}

		if (isCampNumericSensePrecisio(tipus)) {
			return getColumnaInteger();
		}

		if (isCampNumericPrecisio(tipus)) {
			return getColumnaBigDecimal();
		}
		if (isCampDate(tipus)) {
			return getColumnaData();
		}

		throw new PersistWorkException(PersistWorkException.MAP_10,
				PersistWorkException.MAP10_TXT);

	}

	/**
	 * Retornar si es tracta d'un camp de tipus text.
	 * 
	 * @param tipus
	 *            de camp
	 * @return es de tipus text
	 */
	public boolean isCampText(String tipus) {

		return "java.lang.String".equals(tipus);
	}

	/**
	 * Retornar si es tracta d'un camp numric amb precisi.
	 * 
	 * @param tipus
	 *            de camp
	 * @return s numric amb precisi.
	 */
	public boolean isCampNumericPrecisio(String tipus) {

		return ("double".equals(tipus) || "java.lang.Double".equals(tipus)
				|| "java.lang.Float".equals(tipus) || "float".equals(tipus) || "java.math.BigDecimal"
					.equals(tipus));

	}

	/**
	 * Retornar si es tracta d'un camp de tipus data.
	 * 
	 * @param tipus
	 *            de camp
	 * @return s camp data.
	 */
	public boolean isCampDate(String tipus) {

		return ("java.util.GregorianCalendar".equals(tipus) || "java.util.Date"
				.equals(tipus));
	}

	/**
	 * Retornar si es tracta d'un camp de tipus numric sense precisi.
	 * 
	 * @param tipus
	 *            de camp
	 * @return s numric sense precisi.
	 */
	public boolean isCampNumericSensePrecisio(String tipus) {

		return ("int".equals(tipus) || "java.lang.Integer".equals(tipus)
				|| "java.lang.Long".equals(tipus) || "long".equals(tipus));

	}

	/**
	 * Retornar si es tracat d'un valor null en els casos que el tipus de camp
	 * s bsic ( el valor nul s 0 )
	 * 
	 * @param tipus
	 *            de camp
	 * @param object
	 *            objecte
	 * @return s nul tipus bsic
	 */
	public boolean isNullBasicType(String tipus, Object object) {

		if ("int".equals(tipus))
			return (Integer)object ==0;
			
		if ("long".equals(tipus)) 
			return (Long)object ==0;
						
		if ("double".equals(tipus))
			return (Double)object ==0.0;
		
		if ("float".equals(tipus)) 
			return (Float)object ==0.0;
		

		return false;
	}

	/**
	 * Retornar el tipus de columna quan es tracta un bigdecimal.
	 * 
	 * @return tipus de columna base de dades
	 */
	protected abstract String getColumnaBigDecimal();

	/**
	 * Retornar el tipus de columna quan es tracta una data.
	 * 
	 * @return tipus de columna
	 */
	protected abstract String getColumnaData();

	/**
	 * Retornar el tipus de columna quan es tracta de text
	 * 
	 * @return tipus de columna
	 */
	protected abstract String getColumnaText();

	/**
	 * Retornar el tipus de columna quan es tracta d'un enter
	 * 
	 * @return tipus de columna.
	 */
	protected abstract String getColumnaInteger();

	/**
	 * Crear una instncia segons el tipus de motor de base de dades que hagi
	 * configurat a l'entorn per defecte.
	 * 
	 * @return equivalncia de columna
	 * @throws PersistWorkException
	 */
	protected static EquivalenciaColumna getNewEquivalCol()
			throws PersistWorkException {

		String db = ConfiguracioEntorn
				.getValorPropietat(ConfiguracioEntorn.PROPS_BDTIPUS);
		if (ConnexioBD.BD_ORACLE.equals(db)) {
			return new EquivalenciaColOracle();
		} else {
			return new EquivalenciaColMySql();
		}

	}

	/**
	 * Crear una instncia segons el tipus de motor de base de dades que hagi
	 * configurat en el entorn indicat per parmetre
	 * 
	 * @param entorn
	 *            identificador de l'entorn
	 * @return equivalncia de columna.
	 * @throws PersistWorkException
	 */
	protected static EquivalenciaColumna getNewEquivalCol(String entorn)
			throws PersistWorkException {

		String db = ConfiguracioEntorn.getValorPropietat(entorn,
				ConfiguracioEntorn.PROPS_BDTIPUS);
		if (ConnexioBD.BD_ORACLE.equals(db)) {
			return new EquivalenciaColOracle();
		} else {
			return new EquivalenciaColMySql();
		}

	}

	/**
	 * Retornar el valor correcte per a poder-l'ho assignar al camp 
	 * segons el que s'hagi rebut de la base de dades i segons sigui el tipus
	 * de camp de l'objecet.
	 * @param valBD valor rebut de la base de dades
	 * @param tipus tipus de camp a on s'ha de ficar el valor
	 * @return nou valor transformat del tipus correcte
	 */
	protected abstract Object getValor(Object valBD, String tipus)
			throws PersistWorkException;

}
