package com.smg.persistwork.nucli;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import com.smg.persistwork.exception.PersistWorkException;

/**
 * Configuraci de l'entorn del framework. Es poden configurar diferents propietats.
 * Tamb hi ha diferents entorns configurables.
 * @author Sergio Maeso
 * @version 1.0
 */
public class ConfiguracioEntorn {

	
	/**
	 * Id. de l'entorn per defecte
	 */
	private static final String ID_DEFECTE = "ConfiguracioEntorn.ID"; 
	
	/**
	 * Ruta del fitxer per defecte de propietats
	 */
	private static final String RUTA_CONF_DEFECTE = "META-INF/";
	
	
	/**
	 * Nom del fitxer de propietats per defecte
	 */
	protected static final String FIT_CONF_DEFECTE_PROP = "persist-work-conf.properties";
	

	/*
	 * Propietats que accepta el ftixer de propietats.
	 */
	public static String PROPS_BDTIPUS = "persistwork.bd.tipus";
	public static String PROPS_BDUSUARI = "persistwork.bd.usuari";
	public static String PROPS_BDCONTRASENYA = "persistwork.bd.contrasenya";
	public static String PROPS_CADENA = "persistwork.bd.cadenaConnexio";
	public static String PROPS_PACKAGE = "persistwork.bd.package";
	public static String PROPS_ENTITATS = "persistwork.bd.entitats";
	
	/**
	 * Id. de l'entorn
	 */
	private String id;

	/**
	 * Fitxer de propietats
	 */
	private String fitxerPropietats;

	/**
	 * Propietats carregades a partir del fitxer de propietats
	 */
	private Properties props = null;

	/**
	 * Ubicaci dels diferents entorns que es poden configurar.
	 */
	private static Map<String, ConfiguracioEntorn> entorns = new HashMap<String, ConfiguracioEntorn>();

	/**
	 * Retorna el identificador de l'entorn
	 * @return identificador
	 */
	public String getId() {
		return id;
	}

	/**
	 * Retorna un entorn
	 * @param id
	 * @return ConfiguracioEntorn
	 * @throws PersistWorkException 
	 */
	protected static ConfiguracioEntorn getEntorn(String... id) throws PersistWorkException {

		if (id.length == 0){
			if (!entorns.containsKey(ConfiguracioEntorn.ID_DEFECTE)){
				creaConfiguracio();
			}
			return entorns.get(ConfiguracioEntorn.ID_DEFECTE);
		} else{
			if (!entorns.containsKey(id[0])){
				creaConfiguracio(id[0]);
			} 
			return entorns.get(id[0]);
		}			
		
	}

	/**
	 * Retorna el valor d'una propietat d'un entorn diferent al per defecte
	 * @param idEntorn
	 * @param idProperty
	 * @return valor propietat
	 * @throws PersistWorkException 
	 */
	public static String getValorPropietat(String idEntorn, String idProperty) throws PersistWorkException {
		
		ConfiguracioEntorn ce = getEntorn(idEntorn);
		
		return ce.props.getProperty(idProperty);
	}
	
	/**
	 * Retorna el valor d'una propietat de l'entorn per defecte.
	 * @param idProperty
	 * @return valor propietat
	 * @throws PersistWorkException 
	 */
	public static String getValorPropietat(String idProperty) throws PersistWorkException {
		
		ConfiguracioEntorn ce = getEntorn();
			
		return ce.props.getProperty(idProperty);
	}

	/**
	 * Crea la configuraci de l'entorn.
	 * @param valors
	 * @throws PersistWorkException
	 */
	public static void creaConfiguracio(String... valors)
			throws PersistWorkException {

		if (valors.length > 2) {
			throw new PersistWorkException(PersistWorkException.NUC_1,
					PersistWorkException.NUC1_TXT);
		}
		controlEntorns(valors);
	}

	/**
	 * Fa el control dels entorns, per a no tindre entorns repetits.
	 * @param valors	 
	 * @throws PersistWorkException 
	 */
	private static void controlEntorns(String... valors) throws PersistWorkException {
		String id;

		if (entorns == null) {
			entorns = new HashMap<String, ConfiguracioEntorn>();
		}

		if (valors.length == 0) {
			id = ConfiguracioEntorn.ID_DEFECTE;
		} else {
			id = valors[0];
		}
		if (!entorns.containsKey(id)) {
			ConfiguracioEntorn ce = new ConfiguracioEntorn();
			ce.id = id;
			if (valors.length == 2) {
				ce.fitxerPropietats = valors[1];
			} else {
				ce.fitxerPropietats = ConfiguracioEntorn.RUTA_CONF_DEFECTE
						+ ConfiguracioEntorn.FIT_CONF_DEFECTE_PROP;
			}

			entorns.put(ce.id, ce);
			ce.carregaConfiguracio(ce.id);
			
		}
	}

	/**
	 * Carrega la configuraci del fitxer de propietats
	 * @throws PersistWorkException 
	 * 
	 */
	protected  void carregaConfiguracio(String... id) throws PersistWorkException {

		ConfiguracioEntorn en;
		if (id.length > 0) {
			en = getEntorn(id[0]);
		} else {
			en = getEntorn(ConfiguracioEntorn.ID_DEFECTE);
		}

		InputStream fileprops = en.getClass().getClassLoader()
				.getResourceAsStream(en.fitxerPropietats);

		if (fileprops != null) {
			en.props = new Properties();
			try {
				en.props.load(fileprops);
				fileprops.close();
			} catch (IOException e) {
				throw new PersistWorkException(PersistWorkException.NUC_2,
						PersistWorkException.NUC2_TXT);
			}
		}
		
	}
	
	/**
	 * Elimina tots els entorns.
	 * @throws PersistWorkException 
	 * 
	 */
	public static void removeEntorns() throws PersistWorkException {

		entorns.clear();
		
	}
}
