package com.smg.persistwork.nucli;

import com.smg.persistwork.connexio.bd.ConnexioBD;
import com.smg.persistwork.connexio.bd.ConnexioBDMySql;
import com.smg.persistwork.connexio.bd.ConnexioBDOracle;
import com.smg.persistwork.exception.PersistWorkException;
import com.smg.persistwork.model.mapeig.ObjecteMapejat;
import com.smg.persistwork.model.mapeig.RegistreObjectes;
import com.smg.persistwork.model.mapeig.RegistreObjectesFactory;

/**
 * Classe que s'encarrega de les funcionalitats a nivell d'administraci
 * de la base de dades
 * @author Sergio Maeso
 * @version 1.0
 */
public class EsquemaAplicacio {

	/**
	 * Connexi amb la base de dades
	 * @see com.smg.persistwork.connexio.bd.ConnexioBD
	 */
	private ConnexioBD connexio;

	/**
	 * Identificador de l'entorn.
	 */
	private String id;

	/**
	 * Constructor amb entorn per defecte.
	 * @throws PersistWorkException
	 * 
	 */
	public EsquemaAplicacio() throws PersistWorkException {

		ConfiguracioEntorn.creaConfiguracio();

		if (ConnexioBD.BD_ORACLE.equals(ConfiguracioEntorn
				.getValorPropietat(ConfiguracioEntorn.PROPS_BDTIPUS))) {
			this.connexio = new ConnexioBDOracle(
					ConfiguracioEntorn
							.getValorPropietat(ConfiguracioEntorn.PROPS_CADENA),
					ConfiguracioEntorn
							.getValorPropietat(ConfiguracioEntorn.PROPS_BDUSUARI),
					ConfiguracioEntorn
							.getValorPropietat(ConfiguracioEntorn.PROPS_BDCONTRASENYA));
		} else {
			this.connexio = new ConnexioBDMySql(
					ConfiguracioEntorn
							.getValorPropietat(ConfiguracioEntorn.PROPS_CADENA),
					ConfiguracioEntorn
							.getValorPropietat(ConfiguracioEntorn.PROPS_BDUSUARI),
					ConfiguracioEntorn
							.getValorPropietat(ConfiguracioEntorn.PROPS_BDCONTRASENYA));
		}
	}

	/**
	 * Constructor indicant un entorn diferent al de defecte.
	 * @param id
	 * @throws PersistWorkException
	 */
	public EsquemaAplicacio(String id, String fitxer)
			throws PersistWorkException {

		ConfiguracioEntorn.creaConfiguracio(id, fitxer);
		this.id = id;
		if (ConnexioBD.BD_ORACLE.equals(ConfiguracioEntorn.getValorPropietat(
				id, ConfiguracioEntorn.PROPS_BDTIPUS))) {
			connexio = new ConnexioBDOracle(
					ConfiguracioEntorn.getValorPropietat(id,
							ConfiguracioEntorn.PROPS_CADENA),
					ConfiguracioEntorn.getValorPropietat(id,
							ConfiguracioEntorn.PROPS_BDUSUARI),
					ConfiguracioEntorn.getValorPropietat(id,
							ConfiguracioEntorn.PROPS_BDCONTRASENYA));
		} else {
			connexio = new ConnexioBDMySql(
					ConfiguracioEntorn.getValorPropietat(id,
							ConfiguracioEntorn.PROPS_CADENA),
					ConfiguracioEntorn.getValorPropietat(id,
							ConfiguracioEntorn.PROPS_BDUSUARI),
					ConfiguracioEntorn.getValorPropietat(id,
							ConfiguracioEntorn.PROPS_BDCONTRASENYA));
		}
	}

	/**
	 * Crea una taula a la base de daes
	 * @param obj
	 * @throws PersistWorkException
	 */
	public void creaTaula(Object obj) throws PersistWorkException {

		RegistreObjectes reg = RegistreObjectesFactory.getRegistre(id);

		ObjecteMapejat objmap = reg.getObjecte(obj);

		connexio.creaConnexio();
		// creaci taula
		connexio.executaSentencia(objmap.getSqlCreate());

		// creaci clau primria
		if (objmap.getSqlCreateCP() != null)
			connexio.executaSentencia(objmap.getSqlCreateCP());

		// creaci seqncies
		for (String sent : objmap.getCreateSeqCP()) {
			connexio.executaSentencia(sent);
		}

		// creaci claus foraneas
		for (String sent : objmap.getSqlCreateClausForaneas()) {
			connexio.executaSentencia(sent);
		}
		connexio.tancaConnexio();

	}

	/**
	 * Elimina una taula a la base de dades
	 * @param obj
	 * @throws PersistWorkException
	 */
	public void eliminaTaula(Object obj) throws PersistWorkException {
		RegistreObjectes reg = RegistreObjectesFactory.getRegistre(id);
		ObjecteMapejat objmap = reg.getObjecte(obj);
		connexio.creaConnexio();
		// esborrar seqncies
		for (String sent : objmap.getDropSeqCP()) {
			connexio.executaSentencia(sent);
		}
		connexio.executaSentencia(objmap.getSqlDrop());
		connexio.tancaConnexio();
	}

	/**
	 * Crea la base de dades a partir dels noms de les entitats
	 * configurades al fitxer de propietats.
	 * @throws PersistWorkException
	 * 
	 */
	public void creaBaseDades() throws PersistWorkException {

		Object[] objs = getNomEntitats();

		for (Object obj : objs) {
			creaTaula(obj);
		}

	}

	/**
	 * Elimina la base de dades a partir dels nom de les entitats
	 * configurades al fitxer de propietats.
	 * @throws PersistWorkException
	 * 
	 */
	public void eliminaBaseDades() throws PersistWorkException {
		Object[] objs = getNomEntitats();

		for (int i = objs.length - 1; i >= 0; i--) {
			eliminaTaula(objs[i]);
		}
	}

	/**
	 * Retorna els objectes a partir del package i dels noms de les
	 * entitats definides al fitxer de propietats.
	 * @return
	 * @throws PersistWorkException
	 */
	protected Object[] getNomEntitats() throws PersistWorkException {
		String nompackage;
		String entitats;
		String[] classes;
		Object[] objectes;
		if (id != null) {
			nompackage = ConfiguracioEntorn.getValorPropietat(id,
					ConfiguracioEntorn.PROPS_PACKAGE);
			entitats = ConfiguracioEntorn.getValorPropietat(id,
					ConfiguracioEntorn.PROPS_ENTITATS);
		} else {
			nompackage = ConfiguracioEntorn
					.getValorPropietat(ConfiguracioEntorn.PROPS_PACKAGE);
			entitats = ConfiguracioEntorn
					.getValorPropietat(ConfiguracioEntorn.PROPS_ENTITATS);
		}
		classes = entitats.split(",");
		objectes = new Object[classes.length];
		for (int i = 0; i < classes.length; i++) {
			Class<?> c;
			try {
				c = Class.forName(nompackage.trim() + "." + classes[i].trim());
				Object obj = c.newInstance();
				objectes[i] = obj;
			} catch (ClassNotFoundException e) {

				throw new PersistWorkException(PersistWorkException.NUC_3,
						PersistWorkException.NUC3_TXT);
			} catch (InstantiationException e) {

				throw new PersistWorkException(PersistWorkException.NUC_4,
						PersistWorkException.NUC4_TXT);
			} catch (IllegalAccessException e) {

				throw new PersistWorkException(PersistWorkException.NUC_5,
						PersistWorkException.NUC5_TXT);
			} catch (NullPointerException e) {

				throw new PersistWorkException(PersistWorkException.NUC_6,
						PersistWorkException.NUC6_TXT);
			} catch (Exception e) {

				throw new PersistWorkException(PersistWorkException.NUC_7,
						PersistWorkException.NUC7_TXT);
			}

		}

		return objectes;
	}

	/**
	 * Retorna l'identificador
	 * @return
	 */
	protected String getId() {
		return id;
	}

	/**
	 * Retorna la connexi.
	 * @return
	 */
	protected ConnexioBD getConnexio() {
		return connexio;
	}

}
