package com.smg.persistwork.nucli;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import com.smg.persistwork.connexio.bd.ConnexioBD;
import com.smg.persistwork.connexio.bd.ConnexioBDMySql;
import com.smg.persistwork.connexio.bd.ConnexioBDOracle;
import com.smg.persistwork.exception.PersistWorkException;
import com.smg.persistwork.model.annotations.ClauForanea;
import com.smg.persistwork.model.annotations.ClauPrimaria;
import com.smg.persistwork.model.annotations.Columna;
import com.smg.persistwork.model.mapeig.MapeigSql;
import com.smg.persistwork.model.mapeig.ObjecteMapejat;
import com.smg.persistwork.model.mapeig.RegistreObjectes;
import com.smg.persistwork.model.mapeig.RegistreObjectesFactory;
import com.smg.persistwork.utilscerca.Filtre;

/**
 * Manegador Entitats: s'encarrega de realitzar totes les accions
 * sobre la base de dades, menys les d'adminstraci. Amb aquesta classe
 * podem fer insercions, modificacions, esborrar, cerques per clau, per camps,
 * per sql natiu i per fwkql.
 * @author Sergio Maeso
 * @version 1.0
 * 
 */
public class ManegadorEntitat {

	/**
	 * Identificador de l'entorn, en cas que tinguem entorn diferent al de defecte.
	 */
	private String id;
	/**
	 * Connexi amb la base de dades.
	 */
	private ConnexioBD connexio;

	/**
	 * Indica si la transacci est activa
	 */
	private boolean transActiva = false;

	/**
	 * Constructor amb entorn per defecte
	 * @throws PersistWorkException
	 * 
	 */
	protected ManegadorEntitat() throws PersistWorkException {

		ConfiguracioEntorn.creaConfiguracio();

		if (ConnexioBD.BD_ORACLE.equals(ConfiguracioEntorn
				.getValorPropietat(ConfiguracioEntorn.PROPS_BDTIPUS))) {
			this.connexio = new ConnexioBDOracle(
					ConfiguracioEntorn
							.getValorPropietat(ConfiguracioEntorn.PROPS_CADENA),
					ConfiguracioEntorn
							.getValorPropietat(ConfiguracioEntorn.PROPS_BDUSUARI),
					ConfiguracioEntorn
							.getValorPropietat(ConfiguracioEntorn.PROPS_BDCONTRASENYA));
		} else {
			this.connexio = new ConnexioBDMySql(
					ConfiguracioEntorn
							.getValorPropietat(ConfiguracioEntorn.PROPS_CADENA),
					ConfiguracioEntorn
							.getValorPropietat(ConfiguracioEntorn.PROPS_BDUSUARI),
					ConfiguracioEntorn
							.getValorPropietat(ConfiguracioEntorn.PROPS_BDCONTRASENYA));
		}
	}

	/**
	 * Constructor indicant un altre entorn diferent.
	 * @param id
	 * @throws PersistWorkException
	 */
	protected ManegadorEntitat(String id, String fitxer)
			throws PersistWorkException {
		ConfiguracioEntorn.creaConfiguracio(id, fitxer);
		this.id = id;
		if (ConnexioBD.BD_ORACLE.equals(ConfiguracioEntorn.getValorPropietat(
				id, ConfiguracioEntorn.PROPS_BDTIPUS))) {
			connexio = new ConnexioBDOracle(
					ConfiguracioEntorn.getValorPropietat(id,
							ConfiguracioEntorn.PROPS_CADENA),
					ConfiguracioEntorn.getValorPropietat(id,
							ConfiguracioEntorn.PROPS_BDUSUARI),
					ConfiguracioEntorn.getValorPropietat(id,
							ConfiguracioEntorn.PROPS_BDCONTRASENYA));
		} else {
			connexio = new ConnexioBDMySql(
					ConfiguracioEntorn.getValorPropietat(id,
							ConfiguracioEntorn.PROPS_CADENA),
					ConfiguracioEntorn.getValorPropietat(id,
							ConfiguracioEntorn.PROPS_BDUSUARI),
					ConfiguracioEntorn.getValorPropietat(id,
							ConfiguracioEntorn.PROPS_BDCONTRASENYA));
		}
	}

	/**
	 * Inicialitza la transacci. Noms fa falta si es vol treballar amb
	 * transaccions.
	 * @throws PersistWorkException
	 */
	public void inicialitzaTransaccion() throws PersistWorkException {

		connexio.creaConnexio();
		connexio.iniciaTransaccio();
		transActiva = true;
	}
	
	/**
	 * Finalitza la transacci actual.
	 * @throws PersistWorkException
	 */
	public void finalitzaTransaccion() throws PersistWorkException {

		connexio.finalitzaTransaccio();
		transActiva = false;
	}

	/**
	 * Realitza el commit i finalitza la transacci actual.
	 * @throws PersistWorkException
	 */
	public void ferCommit() throws PersistWorkException {

		connexio.finalitzaTransaccio();
		transActiva = false;
	}

	/**
	 * Realitza el rollback i finalitza la transacci actual.
	 * @throws PersistWorkException
	 */
	public void ferRollBack() throws PersistWorkException {
		connexio.rollback();
		transActiva = false;
	}

	/**
	 * Inserta un registre a la base de dades.
	 * @param <T>
	 * @param obj
	 * @throws PersistWorkException
	 */
	public <T> void inserta(T obj) throws PersistWorkException {

		RegistreObjectes reg = RegistreObjectesFactory.getRegistre(id);

		ObjecteMapejat objmap = reg.getObjecte(obj);

		if (!transActiva) {
			connexio.creaConnexio();
		}

		List<Object> valors = new ArrayList<Object>();

		String sbConsulta = getSentencia(obj, objmap, objmap.getSqlInsert(),
				valors, false);

		connexio.executaSentencia(sbConsulta, valors);
		if (objmap.getIncrementLastId()){
			ResultSet res = connexio.executaSentenciaDades("SELECT LAST_INSERT_ID()");
			try {
				if (res.next())
					objmap.setLastId(res, obj);
			} catch (SQLException e) {
				throw new PersistWorkException(PersistWorkException.NUC_8,
						PersistWorkException.NUC8_TXT);
			}
		}
		if (!transActiva) {
			connexio.tancaConnexio();
		}

	}

	/**
	 * Modifica un registre a la base de dades.
	 * @param <T>
	 * @param obj
	 * @throws PersistWorkException
	 */
	public <T> void modifica(T obj) throws PersistWorkException {

		RegistreObjectes reg = RegistreObjectesFactory.getRegistre(id);
		ObjecteMapejat objmap = reg.getObjecte(obj);

		if (!transActiva) {
			connexio.creaConnexio();
		}

		List<Object> valors = new ArrayList<Object>();

		String sbModifica = getSentencia(obj, objmap, objmap.getSqlUpdate(),
				valors, false);

		connexio.executaSentencia(sbModifica, valors);
		if (!transActiva) {
			connexio.tancaConnexio();
		}

	}

	/**
	 * Esborra un registre de la base de dades.
	 * @param <T>
	 * @param obj
	 * @throws PersistWorkException
	 */
	public <T> void esborra(T obj) throws PersistWorkException {

		RegistreObjectes reg = RegistreObjectesFactory.getRegistre(id);
		ObjecteMapejat objmap = reg.getObjecte(obj);

		if (!transActiva) {
			connexio.creaConnexio();
		}

		List<Object> valors = new ArrayList<Object>();

		String sbEsborra = getSentencia(obj, objmap, objmap.getSqlDelete(),
				valors, true);

		connexio.executaSentencia(sbEsborra, valors);
		if (!transActiva) {
			connexio.tancaConnexio();
		}
	}

	/**
	 * Retorna un registre a partir de la seva clau primria
	 * @param <T>
	 * @param obj
	 * @return resultat registre
	 * @throws PersistWorkException
	 */
	public <T> T consultaPerClau(T obj) throws PersistWorkException {

		RegistreObjectes reg = RegistreObjectesFactory.getRegistre(id);
		ObjecteMapejat objmap = reg.getObjecte(obj);
		T ret = null;
		
		connexio.creaConnexio();
		
		List<Object> valors = new ArrayList<Object>();
		String sbConsulta = getSentencia(obj, objmap, objmap.getSqlSelect(),
				valors, true);

		ResultSet rs = connexio.executaSentenciaDades(sbConsulta, valors);
		try {
			if (rs.next()){
				ret = objmap.getInstance(rs);
			}
				
		} catch (SQLException e) {
			throw new PersistWorkException(PersistWorkException.NUC_8,
					PersistWorkException.NUC8_TXT);
		}

		connexio.tancaConnexio();

		return ret;

	}
	

	/** Realitza una cerca fent servir l'objecte per a filtrar.
	 * @param <T>
	 * @param obj
	 * @return resultat registres
	 * @throws PersistWorkException
	 */
	public <T> List<T> cerca(T obj) throws PersistWorkException {

		RegistreObjectes reg = RegistreObjectesFactory.getRegistre(id);
		ObjecteMapejat objmap = reg.getObjecte(obj);
		List<T> ret = new ArrayList<T>();
		
		connexio.creaConnexio();
		
		List<Object> valors = new ArrayList<Object>();
			
		String sbConsulta = getSentencia(obj, objmap,objmap.getSqlCondCerca(obj),
				valors, false);

		ResultSet rs = connexio.executaSentenciaDades(sbConsulta, valors);
		try {
			while (rs.next()){
				ret.add((T) objmap.getInstance(rs));
			}
				
		} catch (SQLException e) {
			throw new PersistWorkException(PersistWorkException.NUC_8,
					PersistWorkException.NUC8_TXT);
		}

		connexio.tancaConnexio();

		return ret;

	}
	
	/** Realitza una cerca fent servir l'objecte per a filtrar.
	 * @param <T>
	 * @param obj
	 * @return resultat registres
	 * @throws PersistWorkException
	 */
	public <T> List<T> cercaTots(T obj) throws PersistWorkException {

		RegistreObjectes reg = RegistreObjectesFactory.getRegistre(id);
		ObjecteMapejat objmap = reg.getObjecte(obj);
		List<T> ret = new ArrayList<T>();
		
		connexio.creaConnexio();
		
		List<Object> valors = new ArrayList<Object>();
			
		ResultSet rs = connexio.executaSentenciaDades(objmap.getSqlCamps(), valors);
		try {
			while (rs.next()){
				ret.add((T) objmap.getInstance(rs));
			}
				
		} catch (SQLException e) {
			throw new PersistWorkException(PersistWorkException.NUC_8,
					PersistWorkException.NUC8_TXT);
		}

		connexio.tancaConnexio();

		return ret;

	}
	

	/**
	 * Realitza una cerca d'objecte fent servir la utilitat dels filtres.
	 * @param <T>
	 * @param obj
	 * @param filtre
	 * @return resultat registres
	 * @throws PersistWorkException
	 */
	public <T> List<T> cerca(T obj, Filtre filtre) throws PersistWorkException {

		RegistreObjectes reg = RegistreObjectesFactory.getRegistre(id);
		ObjecteMapejat objmap = reg.getObjecte(obj);
		List<T> ret = new ArrayList<T>();
		
		connexio.creaConnexio();
				
		StringBuilder sb = new StringBuilder();
		sb.append(objmap.getSqlCamps());
		
		String sbConsulta = getSentenciaFiltre(objmap, filtre);
		sb.append(sbConsulta);
		ResultSet rs = connexio.executaSentenciaDades(sb.toString());
		try {
			while (rs.next()){
				ret.add((T) objmap.getInstance(rs));
			}
				
		} catch (SQLException e) {
			throw new PersistWorkException(PersistWorkException.NUC_8,
					PersistWorkException.NUC8_TXT);
		}

		connexio.tancaConnexio();

		return ret;

	}
	

	/**
	 * Retorna la sentncia en cas que s'hagi fet servir filtres
	 * @param objmap
	 * @param filtre
	 * @return sentncia
	 */
	private String getSentenciaFiltre(ObjecteMapejat objmap, Filtre filtre) {
	
		StringBuilder sentencia = new StringBuilder();
				
		String sql = filtre.getSentencia();
		
		int ultimPos = 0;
		while (ultimPos < sql.length()) {

			int posI = sql.indexOf("@#", ultimPos);

			if (posI == -1) {
				sentencia.append(sql.substring(ultimPos));
				ultimPos = sql.length();
			} else {

				sentencia.append(sql.substring(ultimPos, posI));
				ultimPos = posI + 2;

				int posF = sql.indexOf("@#", ultimPos);
				String stTractar = sql.substring(ultimPos, posF);
				ultimPos = posF + 2;
				
				sentencia.append(objmap.getNomCamp(stTractar.toUpperCase()));							
			}

		}
		return sentencia.toString();
	}

	/**
	 * Realitza una consulta nativa sql.
	 * @param <T>
	 * @param consulta
	 * @param obj
	 * @param map
	 * @return llista resultats
	 * @throws PersistWorkException
	 */
	public <T> List<T> consultaSqlNativa(String consulta, T obj, MapeigSql<T> map) throws PersistWorkException {
	
		List<T> ret = new ArrayList<T>();
		
		connexio.creaConnexio();

		ResultSet rs = connexio.executaSentenciaDades(consulta);
		try {
			while (rs.next()){
				ret.add((T) map.getInstance(rs));
			}
				
		} catch (SQLException e) {
			throw new PersistWorkException(PersistWorkException.NUC_8,
					PersistWorkException.NUC8_TXT);
		}

		connexio.tancaConnexio();

		return ret;

	}
	
	
	/**
	 * Realitzauna consulta amb lleguatge Fwkql
	 * @param <T>
	 * @param consulta
	 * @return llista registres
	 * @throws PersistWorkException
	 */
	public <T> List<T> consultaFwkql(String consulta, List<Object> valors) throws PersistWorkException {

		List<T> ret = new ArrayList<T>();
		RegistreObjectes reg = RegistreObjectesFactory.getRegistre(id);		
		FwkInterpreter fwkinter = new FwkInterpreter(consulta);
				
		ObjecteMapejat objmap = reg.getObjecte(fwkinter.getObjecteSentencia(id));
				
		String sentcamps = objmap.getSqlCamps();		
		String sentCond = objmap.getsqlCondTrad(fwkinter.tradueix(), "");
		
		connexio.creaConnexio();
					
		ResultSet rs = connexio.executaSentenciaDades(sentcamps+sentCond, valors);
		try {
			while (rs.next()){
				ret.add((T) objmap.getInstance(rs));
			}
				
		} catch (SQLException e) {
			throw new PersistWorkException(PersistWorkException.NUC_8,
					PersistWorkException.NUC8_TXT);
		}

		connexio.tancaConnexio();

		return ret;

	}

	/**
	 * Retorna la sentncia definitiva a executar segons 
	 * els valors introduits
	 * @param <T>
	 * @param obj
	 * @param sqlEsborra
	 * @param valors
	 * @return sentncia
	 * @throws PersistWorkException
	 */
	private <T> String getSentencia(T obj, ObjecteMapejat objmap, String sql,
			List<Object> valors, boolean controlNull)
			throws PersistWorkException {

		StringBuilder sentencia = new StringBuilder();
		int ultimPos = 0;
		while (ultimPos < sql.length()) {

			int posI = sql.indexOf("@", ultimPos);

			if (posI == -1) {
				sentencia.append(sql.substring(ultimPos));
				ultimPos = sql.length();
			} else {

				sentencia.append(sql.substring(ultimPos, posI));
				ultimPos = posI + 1;

				int posF = sql.indexOf("@", ultimPos);
				String stTractar = sql.substring(ultimPos, posF);
				ultimPos = posF + 1;

				if (stTractar.contains("$")) {
					// s'ha de cridar a la sentencia, fiquem interrogant
					// i fiquem el resultat al camp.
					int posCI = stTractar.indexOf("#");

					String sent = stTractar.substring(0, posCI)
							.replaceAll("\\$", " ").trim();
					String nomCamp = stTractar.substring(posCI)
							.replaceAll("#", " ").trim();
					ResultSet rs = connexio.executaSentenciaDades(sent);

					try {
						if (rs.next()) {
							sentencia.append(rs.getObject(1));
							objmap.informaValor(obj, rs.getObject(1), nomCamp);
						}
					} catch (SQLException e) {
						throw new PersistWorkException(PersistWorkException.NUC_8,
								PersistWorkException.NUC8_TXT);
					} catch (SecurityException e) {
						throw new PersistWorkException(PersistWorkException.NUC_5,
								PersistWorkException.NUC5_TXT);

					} catch (IllegalArgumentException e) {
						throw new PersistWorkException(PersistWorkException.NUC_5,
								PersistWorkException.NUC5_TXT);
					}

				} else {

					//fiquem ? i fiquem el valor									
					
					try {
						Field camp = objmap.getCamp(stTractar);

						if (camp.isAnnotationPresent(ClauForanea.class)) {
							String nomMetode = "get"
									+ camp.getName().substring(0, 1)
											.toUpperCase()
									+ camp.getName().substring(1);
							String nomMetodeCF = null;
							for (Field f2 : camp.getType().getDeclaredFields()) {

								if (f2.isAnnotationPresent(ClauPrimaria.class)
										&& f2.isAnnotationPresent(Columna.class)) {
									nomMetodeCF = "get"
											+ f2.getName().substring(0, 1)
													.toUpperCase()
											+ f2.getName().substring(1);
								}
							}

							Method mthd = obj.getClass().getDeclaredMethod(
									nomMetode);
							Object objCF = mthd.invoke(obj);
							Method mthdCF = objCF.getClass().getDeclaredMethod(
									nomMetodeCF);
								
							
							
							if (mthdCF.invoke(objCF)==null && controlNull)
								throw new PersistWorkException(PersistWorkException.NUC_12,
										PersistWorkException.NUC12_TXT);
							else{
								if (mthdCF.invoke(objCF) == null) {								
									sentencia.append("NULL");
								} else {
									sentencia.append("?");
									valors.add(mthdCF.invoke(objCF));
								}
								
							}
								

						} else {

							String nomMetode = "get"
									+ camp.getName().substring(0, 1)
											.toUpperCase()
									+ camp.getName().substring(1);
							Method mthd = obj.getClass().getDeclaredMethod(
									nomMetode);
																				
							if (mthd.invoke(obj)==null && controlNull)
								throw new PersistWorkException(PersistWorkException.NUC_12,
										PersistWorkException.NUC12_TXT);
							else{
								if (mthd.invoke(obj) == null) {								
									sentencia.append("NULL");
								} else {
									sentencia.append("?");
									valors.add(mthd.invoke(obj));
								}
							}
								

						}
					} catch (SecurityException e) {
						throw new PersistWorkException(PersistWorkException.NUC_8,
								PersistWorkException.NUC8_TXT);
					} catch (NoSuchMethodException e) {
						throw new PersistWorkException(PersistWorkException.NUC_9,
								PersistWorkException.NUC9_TXT);
					} catch (IllegalArgumentException e) {
						throw new PersistWorkException(PersistWorkException.NUC_10,
								PersistWorkException.NUC10_TXT);
					} catch (IllegalAccessException e) {
						throw new PersistWorkException(PersistWorkException.NUC_5,
								PersistWorkException.NUC5_TXT);
					} catch (InvocationTargetException e) {
						throw new PersistWorkException(PersistWorkException.NUC_11,
								PersistWorkException.NUC11_TXT);
					}

				}
			}

		}
		return sentencia.toString();
	}

}
