package com.smg.persistwork.utilscerca;

import java.util.List;

import com.smg.persistwork.exception.PersistWorkException;

/**
 * Defineix restriccions per a afegir a un filtre
 * @author Sergio Maeso
 * @version 1.0
 */
public class Restriccio {

	/**
	 * Nom del camp 
	 */
	private String nomCamp;

	/**
	 * Valor del camp
	 */
	private Object valor;

	/**
	 * Tipus de restricci que fem al camp i valor
	 */
	private TipusRestriccio tipus;

	/**
	 * Valors afectats a la restricci
	 */
	private List<Object> valors;

	/**
	 * Constructor
	 * @param nomCamp
	 * @param valor
	 */
	private Restriccio(String nomCamp, Object valor) {
		super();
		this.nomCamp = nomCamp;
		this.valor = valor;
	}

	/**
	 * Constructor
	 * @param nomCamp
	 */
	private Restriccio(String nomCamp) {
		super();
		this.nomCamp = nomCamp;
	}

	/**
	 * Constructor
	 * @param nomCamp
	 * @param valors
	 */
	private Restriccio(String nomCamp, List<Object> valors) {
		super();
		this.nomCamp = nomCamp;
		this.valors = valors;
	}

	/**
	 * Crea una restricci de tipus igual
	 * NOMCAMP = VALORCAMP
	 * @param nomCamp
	 * @param valorCamp
	 * @return Restriccio
	 * @throws PersistWorkException 
	 */
	public static Restriccio igual(String nomCamp, Object valorCamp) throws PersistWorkException {

		Restriccio rest = new Restriccio(nomCamp, valorCamp);
		rest.tipus = TipusRestriccio.IGUAL;
		if (valorCamp == null){
			throw new PersistWorkException(4,"lalla");
		}
		return rest;
	}

	/**
	 * Crea una restricci de tipus no igual
	 * NOMCAMP != VALORCAMP
	 * @param nomCamp
	 * @param valorCamp
	 * @return Restriccio
	 */
	public static Restriccio noIgual(String nomCamp, Object valorCamp) {

		Restriccio rest = new Restriccio(nomCamp, valorCamp);
		rest.tipus = TipusRestriccio.DIFERENT;

		return rest;
	}

	/**
	 * Crea una restricci de tipus major que
	 * NOMCAMP > VALORQUE
	 * @param nomCamp
	 * @param valorCamp
	 * @return Restriccio
	 */
	public static Restriccio majorque(String nomCamp, Object valorCamp) {

		Restriccio rest = new Restriccio(nomCamp, valorCamp);
		rest.tipus = TipusRestriccio.MAJORQUE;

		return rest;
	}

	/**
	 * Crea una restricci de tipus menor que
	 * NOMCAMP < VALORCAMP
	 * @param nomCamp
	 * @param valorCamp
	 * @return Restriccio
	 */
	public static Restriccio menorque(String nomCamp, Object valorCamp) {

		Restriccio rest = new Restriccio(nomCamp, valorCamp);
		rest.tipus = TipusRestriccio.MENORQUE;

		return rest;
	}

	/**
	 * Crea una restricci de tipus igual major que
	 * NOMCAMP >= VALORCAMP
	 * @param nomCamp
	 * @param valorCamp
	 * @return Restriccio
	 */
	public static Restriccio igualmajorque(String nomCamp, Object valorCamp) {

		Restriccio rest = new Restriccio(nomCamp, valorCamp);
		rest.tipus = TipusRestriccio.IGUALMAJORQUE;

		return rest;
	}

	/**
	 * Crea una restricci de tipus igual menor que
	 * NOMCAMP <= VALORCAMP
	 * @param nomCamp
	 * @param valorCamp
	 * @return Restriccio
	 */
	public static Restriccio igualmenorque(String nomCamp, Object valorCamp) {

		Restriccio rest = new Restriccio(nomCamp, valorCamp);
		rest.tipus = TipusRestriccio.IGUALMENORQUE;

		return rest;
	}

	/**
	 * Crea una restricci de tipus es nul
	 * NOMCAMP IS NULL
	 * @param nomCamp
	 * @return Restriccio
	 */
	public static Restriccio nul(String nomCamp) {

		Restriccio rest = new Restriccio(nomCamp);
		rest.tipus = TipusRestriccio.NUL;

		return rest;
	}

	/**
	 * Crea una restricci de tipus no es nul
	 * NOMCAMP IS NOT NULL
	 * @param nomCamp
	 * @return Restriccio
	 */
	public static Restriccio nonul(String nomCamp) {

		Restriccio rest = new Restriccio(nomCamp);
		rest.tipus = TipusRestriccio.NONUL;

		return rest;
	}

	/**
	 * Crea una restricci de tipus dins uns valors
	 * NOMCAMP IN ( LISTVALORS )
	 * @param nomCamp
	 * @param valorsCamp
	 * @return Restriccio
	 */
	public static Restriccio dinsunsvalors(String nomCamp,
			List<Object> valorsCamp) {

		Restriccio rest = new Restriccio(nomCamp, valorsCamp);
		rest.tipus = TipusRestriccio.DINTREUNSVALORS;

		return rest;
	}

	/**
	 * Crea una restricci de tipus fora uns valors
	 * NOMCAMP NOT IN ( LISTVALORS )
	 * @param nomCamp
	 * @param valorsCamp
	 * @return Restriccio
	 */
	public static Restriccio nodinsunsvalors(String nomCamp,
			List<Object> valorsCamp) {

		Restriccio rest = new Restriccio(nomCamp, valorsCamp);
		rest.tipus = TipusRestriccio.FORAUNSVALORS;

		return rest;
	}

	/**
	 * Retorna la sentncia associada a aquesta restricci
	 * @return sentncia associada a la restricci
	 */
	protected String getSentencia() {

		String sentence = null;
	
		switch (tipus) {

		case IGUAL:
			sentence = "@#"+ nomCamp + "@#=" + getValor(valor);
			break;
		case DIFERENT:
			sentence = "@#"+ nomCamp + "@#!=" + getValor(valor);
			break;			
		case NUL:
			sentence = "@#"+ nomCamp + "@# IS NULL";
			break;
		case NONUL:
			sentence = "@#"+ nomCamp + "@# IS NOT NULL";
			break;
		case MAJORQUE:
			sentence = "@#"+ nomCamp + "@# > " + getValor(valor);
			break;
		case MENORQUE:
			sentence = "@#"+ nomCamp + "@#<" + getValor(valor);
			break;			
		case IGUALMAJORQUE:
			sentence = "@#"+ nomCamp + "@# >= " + getValor(valor);
			break;
		case IGUALMENORQUE:
			sentence = "@#"+ nomCamp + "@# <= " + getValor(valor);
			break;
		case DINTREUNSVALORS:
			sentence = "@#"+ nomCamp + "@# IN (";
			boolean ficaComa = false;
			for (Object val :valors){
				if (ficaComa){
					sentence+=",";
				} else {
					ficaComa=true;
					
				}
				sentence+=getValor(val);
			}
			sentence+=")";
			
			break;
		case FORAUNSVALORS:
			sentence = "@#"+ nomCamp + "@# NOT IN (";
			ficaComa = false;
			for (Object val :valors){
				if (ficaComa){
					sentence+=",";
				} else {
					ficaComa=true;
					
				}
				sentence+=val;
			}
			sentence+=")";
			
			break;
		default:
			break;
		}

		return sentence;
	}
	
	/**
	 * Retorna el valor amb apostrfs en cas que hagi apostrofs i
	 * l'objecte sigui un string.
	 * @param val
	 * @return valor
	 */
	private String getValor(Object val){
		
		
		if (val instanceof String){
			return "'"+ val.toString().replace("'", "''") + "'";
		}
		return val.toString();
	}
}
